EXTERN LoadLibrary    ; in DLL.asm
EXTERN GetProcAddress ; in DLL.asm
EXTERN X2C_HALT       ; in X2C.asm

GLOBAL InOut_BEGIN
GLOBAL InOut_Write
GLOBAL InOut_WriteLn
GLOBAL InOut_WriteInt
GLOBAL InOut_WriteCard
GLOBAL InOut_WriteString
GLOBAL InOut_Read
GLOBAL InOut_ReadCard
GLOBAL InOut_ReadString

READ_BUFFER_SIZE equ 80

SECTION .text
; ---------------------------------------------------------------------------- ;
InOut_BEGIN:
        push   ebx
        push   esi
        push   edi

        push   dword sz_console
        call   LoadLibrary
        mov    ecx, eax
        mov    ebx, GetProcAddress

        push   ecx
        push   dword sz_con_init
        call   ebx
        mov    [con_init], eax

        push   ecx
        push   dword sz_con_printf
        call   ebx
        mov    [con_printf], eax

        push   ecx
        push   dword sz_con_gets
        call   ebx
        mov    [con_gets], eax

        push   dword [32]
        push   -1
        push   -1
        push   -1
        push   -1
        call   [con_init]

        pop    edi
        pop    esi
        pop    ebx

        ret
; ---------------------------------------------------------------------------- ;
; PROCEDURE WriteString(v: ARRAY OF CHAR);
InOut_WriteString:
        push   ebx
        push   esi
        push   edi

        push   dword 0
        push   dword [esp + 4 + 4*4]
        call   [con_printf]
        add    esp, 8

        pop    edi
        pop    esi
        pop    ebx

        ret    8
; ---------------------------------------------------------------------------- ;
; PROCEDURE WriteInt(v: INTEGER;  w: CARDINAL);
InOut_WriteInt:
        push   ebx
        push   esi
        push   edi

        push   dword [esp + 4 + 3*4]
        push   dword [esp + 8 + 4*4]
        push   dword sz_format_int
        call   [con_printf]
        add    esp, 12

        pop    edi
        pop    esi
        pop    ebx

        ret    8
; ---------------------------------------------------------------------------- ;
; PROCEDURE WriteLn;
InOut_WriteLn:
        push   dword 0
        push   dword sz_end_of_line
        call   InOut_WriteString
        ret
; ---------------------------------------------------------------------------- ;
; PROCEDURE Write(v: CHAR);
InOut_Write:
        push   ebx
        push   esi
        push   edi

        push   dword [esp + 4 + 3*4]
        push   dword sz_format_char
        call   [con_printf]
        add    esp, 8

        pop    edi
        pop    esi
        pop    ebx

        ret    4
; ---------------------------------------------------------------------------- ;
; PROCEDURE WriteCard(v: CARDINAL; w: CARDINAL);
InOut_WriteCard:
        push   ebx
        push   esi
        push   edi

        push   dword [esp + 4 + 3*4]
        push   dword [esp + 8 + 4*4]
        push   dword sz_format_card
        call   [con_printf]
        add    esp, 12

        pop    edi
        pop    esi
        pop    ebx

        ret    8
; ---------------------------------------------------------------------------- ;
; PROCEDURE Read(VAR v: CHAR);
InOut_Read:
        push   ebx
        push   esi
        push   edi

        mov    esi, [PtrInReadBuffer]
        lodsb
        test   al, al
        jnz    .read_buffer_not_empty

        mov    [PtrInReadBuffer], dword ReadBuffer

        push   dword READ_BUFFER_SIZE
        push   dword ReadBuffer
        call   [con_gets]
        test   eax, eax
        jz     X2C_HALT

        mov    esi, ReadBuffer
        lodsb

.read_buffer_not_empty:
        mov    edi, [esp + 4 + 3*4]
        stosb

        inc    dword [PtrInReadBuffer]

        pop    edi
        pop    esi
        pop    ebx

        ret    4
; ---------------------------------------------------------------------------- ;
; PROCEDURE ReadCard(VAR v: CARDINAL);
InOut_ReadCard:
        push   ebx
        push   esi
        push   edi

        mov    ebx, [esp + 4 + 3*4]

        push   ebp
        mov    ebp, esp
        sub    esp, 16

        lea     edi, [esp]
        mov     ecx, 16
        xor     al, al
        rep stosb

        push    dword 16
        lea     esi, [esp - 1*4]
        push    esi
        call    InOut_ReadString

        xor     ecx, ecx
        xor     eax, eax
.next:
        lodsb
        sub    al, 48
        cmp    al, 9
        jnbe   .done
        lea    ecx, [ecx + ecx * 4]
        lea    ecx, [eax + ecx * 2]
        jmp    .next
.done:

        mov    [ebx], ecx

        mov    esp, ebp
        pop    ebp

        pop    edi
        pop    esi
        pop    ebx

        ret    4
; ---------------------------------------------------------------------------- ;
; PROCEDURE ReadString(VAR v: ARRAY OF CHAR);
InOut_ReadString:
        push   ebx
        push   esi
        push   edi

        mov    esi, [PtrInReadBuffer]
        lodsb
        test   al, al
        jnz    .read_buffer_not_empty

        mov    [PtrInReadBuffer], dword ReadBuffer

        push   dword READ_BUFFER_SIZE
        push   dword ReadBuffer
        call   [con_gets]
        test   eax, eax
        jz     X2C_HALT

        mov    esi, ReadBuffer
        lodsb

.read_buffer_not_empty:
        mov    edi, [esp + 4 + 3*4]
        mov    ecx, [esp + 8 + 3*4]
.skip:
        inc    dword [PtrInReadBuffer]
        jcxz   .done
        test   al, al
        jz     .done
        cmp    al, " "
        jnbe   .next
        dec    ecx
        lodsb
        jmp    .skip
.next:
        jcxz   .done
        stosb
        cmp    al, " "
        jbe    .done
        dec    ecx
        lodsb
        inc    dword [PtrInReadBuffer]
        jmp    .next
.done:
        mov    [edi], byte 0

        pop    edi
        pop    esi
        pop    ebx

        ret    8


SECTION .data
; ---------------------------------------------------------------------------- ;
sz_end_of_line      db 10,0
sz_format_int       db "%*ld",0
sz_format_char      db "%c",0
sz_format_card      db "%*u",0
; ---------------------------------------------------------------------------- ;
sz_con_init         db "con_init",0
sz_con_printf       db "con_printf",0
sz_con_gets         db "con_gets",0
sz_console          db "/sys/lib/console.obj",0
; ---------------------------------------------------------------------------- ;
con_init            dd 0
con_printf          dd 0
con_gets            dd 0
; ---------------------------------------------------------------------------- ;
PtrInReadBuffer     dd ReadBuffer
ReadBuffer:       resb READ_BUFFER_SIZE

